@@ -47,8 +47,8 @@ impl<'boot> Pointer<'boot> {
 
     /// Event to be used with `BootServices::wait_for_event()` in order to wait
     /// for input from the pointer device
-    pub fn wait_for_input_event(&self) -> Event {
-        self.wait_for_input
+    pub fn wait_for_input_event(&self) -> &Event {
+        &self.wait_for_input
     }
 
     /// Returns a reference to the pointer device information.
