@@ -14,7 +14,6 @@ use core::fmt::{Debug, Formatter};
 use core::mem;
 use core::mem::MaybeUninit;
 use core::{fmt, ptr};
-
 /// Contains pointers to all of the runtime services.
 ///
 /// This table, and the function pointers it contains are valid
@@ -27,7 +26,7 @@ pub struct RuntimeServices {
     set_time: unsafe extern "efiapi" fn(time: &Time) -> Status,
     // Skip some useless functions.
     _pad: [usize; 2],
-    set_virtual_address_map: unsafe extern "efiapi" fn(
+    pub(crate) set_virtual_address_map: unsafe extern "efiapi" fn(
         map_size: usize,
         desc_size: usize,
         desc_version: u32,
@@ -92,31 +91,14 @@ impl RuntimeServices {
         (self.set_time)(time).into()
     }
 
-    /// Changes the runtime addressing mode of EFI firmware from physical to virtual.
-    ///
-    /// # Safety
-    ///
-    /// Setting new virtual memory map is unsafe and may cause undefined behaviors.
-    pub unsafe fn set_virtual_address_map(&self, map: &mut [MemoryDescriptor]) -> Result {
-        // Unsafe Code Guidelines guarantees that there is no padding in an array or a slice
-        // between its elements if the element type is `repr(C)`, which is our case.
-        //
-        // See https://rust-lang.github.io/unsafe-code-guidelines/layout/arrays-and-slices.html
-        let map_size = core::mem::size_of_val(map);
-        let entry_size = core::mem::size_of::<MemoryDescriptor>();
-        let entry_version = crate::table::boot::MEMORY_DESCRIPTOR_VERSION;
-        let map_ptr = map.as_mut_ptr();
-        (self.set_virtual_address_map)(map_size, entry_size, entry_version, map_ptr).into()
-    }
-
     /// Get the size (in bytes) of a variable. This can be used to find out how
     /// big of a buffer should be passed in to `get_variable`.
-    pub fn get_variable_size(&self, name: &CStr16, vendor: &Guid) -> Result<usize> {
+    pub fn get_variable_size(&self, name: &CStr16, vendor: &VariableVendor) -> Result<usize> {
         let mut data_size = 0;
         let status = unsafe {
             (self.get_variable)(
                 name.as_ptr(),
-                vendor,
+                &vendor.0,
                 ptr::null_mut(),
                 &mut data_size,
                 ptr::null_mut(),
@@ -139,7 +121,7 @@ impl RuntimeServices {
     pub fn get_variable<'a>(
         &self,
         name: &CStr16,
-        vendor: &Guid,
+        vendor: &VariableVendor,
         buf: &'a mut [u8],
     ) -> Result<(&'a [u8], VariableAttributes)> {
         let mut attributes = VariableAttributes::empty();
@@ -147,7 +129,7 @@ impl RuntimeServices {
         unsafe {
             (self.get_variable)(
                 name.as_ptr(),
-                vendor,
+                &vendor.0,
                 &mut attributes,
                 &mut data_size,
                 buf.as_mut_ptr(),
@@ -189,7 +171,10 @@ impl RuntimeServices {
                         break;
                     };
 
-                    all_variables.push(VariableKey { name, vendor });
+                    all_variables.push(VariableKey {
+                        name,
+                        vendor: VariableVendor(vendor),
+                    });
                 }
                 Status::BUFFER_TOO_SMALL => {
                     // The name buffer passed in was too small, resize it to be
@@ -220,12 +205,19 @@ impl RuntimeServices {
     pub fn set_variable(
         &self,
         name: &CStr16,
-        vendor: &Guid,
+        vendor: &VariableVendor,
         attributes: VariableAttributes,
         data: &[u8],
     ) -> Result {
         unsafe {
-            (self.set_variable)(name.as_ptr(), vendor, attributes, data.len(), data.as_ptr()).into()
+            (self.set_variable)(
+                name.as_ptr(),
+                &vendor.0,
+                attributes,
+                data.len(),
+                data.as_ptr(),
+            )
+            .into()
         }
     }
 
@@ -482,14 +474,30 @@ bitflags! {
     }
 }
 
-/// Vendor GUID used to access global variables.
-pub const GLOBAL_VARIABLE: Guid = Guid::from_values(
-    0x8be4df61,
-    0x93ca,
-    0x11d2,
-    0xaa0d,
-    [0x00, 0xe0, 0x98, 0x03, 0x2b, 0x8c],
-);
+newtype_enum! {
+    /// Variable vendor GUID. This serves as a namespace for variables to
+    /// avoid naming conflicts between vendors. The UEFI specification
+    /// defines some special values, and vendors will define their own.
+    pub enum VariableVendor: Guid => {
+        /// Used to access global variables.
+        GLOBAL_VARIABLE = Guid::from_values(
+            0x8be4df61,
+            0x93ca,
+            0x11d2,
+            0xaa0d,
+            0x00e098032b8c,
+        ),
+
+        /// Used to access EFI signature database variables.
+        IMAGE_SECURITY_DATABASE = Guid::from_values(
+            0xd719b2cb,
+            0x3d3a,
+            0x4596,
+            0xa3bc,
+            0xdad00e67656f,
+        ),
+    }
+}
 
 /// Unique key for a variable.
 #[cfg(feature = "exts")]
@@ -497,7 +505,7 @@ pub const GLOBAL_VARIABLE: Guid = Guid::from_values(
 pub struct VariableKey {
     name: Vec<u16>,
     /// Unique identifier for the vendor.
-    pub vendor: Guid,
+    pub vendor: VariableVendor,
 }
 
 #[cfg(feature = "exts")]
@@ -520,10 +528,10 @@ impl fmt::Display for VariableKey {
 
         write!(f, ", vendor: ")?;
 
-        if self.vendor == GLOBAL_VARIABLE {
+        if self.vendor == VariableVendor::GLOBAL_VARIABLE {
             write!(f, "GLOBAL_VARIABLE")?;
         } else {
-            write!(f, "{}", self.vendor)?;
+            write!(f, "{}", self.vendor.0)?;
         }
 
         write!(f, " }}")
